/*
 * Souffle - A Datalog Compiler
 * Copyright (c) 2017, The Souffle Developers. All rights reserved
 * Licensed under the Universal Permissive License v 1.0 as shown at:
 * - https://opensource.org/licenses/UPL
 * - <souffle root>/licenses/SOUFFLE-UPL.txt
 */

#pragma once

#include <string>
#include <vector>
#include <termios.h>
#include <unistd.h>

/*
 * A class that reads user input a char at a time allowing for tab completion and history to be implemented
 * TODO: link live reader and this class for the warning message that the live reader has finished to be
 * displayed properly
 * TODO: consider: only display history based on current input
 * - eg. history is ["rel R1", "rul N1.1", "graph R1 tot_t"] user types "rel" and presses up, will show "rel
 * R1" only
 */
class InputReader {
private:
    std::string prompt = "Input: ";
    std::vector<std::string> tab_completion;
    std::vector<std::string> history;
    std::string output;
    char current_char = 0;
    size_t cursor_pos = 0;
    size_t hist_pos = 0;
    size_t tab_pos = 0;
    bool in_tab_complete = false;
    bool in_history = false;
    std::string original_hist_val;
    std::string current_hist_val;
    std::string current_tab_val;
    std::string original_tab_val;
    std::vector<std::string> current_tab_completes;
    size_t original_hist_cursor_pos = 0;

public:
    InputReader() {
        clearTabCompletion();
        clearHistory();
    }

    void getch();
    std::string getInput();
    void setPrompt(std::string prompt);
    void appendTabCompletion(std::vector<std::string> commands);
    void appendTabCompletion(std::string command);
    void clearTabCompletion();
    void clearHistory();
    void tabComplete();
    void addHistory(std::string hist);
    void historyUp();
    void historyDown();
    void moveCursor(char direction);
    void moveCursorRight();
    void moveCursorLeft();
    void backspace();
    void clearPrompt(size_t text_len);
    void showFullText(const std::string& text);
};
